package geniusweb.protocol.session;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.references.Parameters;
import geniusweb.references.PartyRef;
import geniusweb.references.PartyWithParameters;
import geniusweb.references.ProfileRef;
import tudelft.utilities.junit.GeneralTests;

public class TeamInfoTest extends GeneralTests<TeamInfo> {

	private TeamInfo team1 = new TeamInfo(Collections.emptyList());
	private TeamInfo team1a = new TeamInfo(Collections.emptyList());
	private PartyRef partyref;
	private Parameters parameters = new Parameters();;
	private ProfileRef profileref;
	private TeamInfo team2, team2a;
	private String asString1 = "{\"TeamInfo\":{\"parties\":[{\"party\":{\"partyref\":\"party1\",\"parameters\":{}},\"profile\":\"profile1\"}]}}";

	@Before
	public void before() throws URISyntaxException {
		partyref = new PartyRef("party1");
		profileref = new ProfileRef("profile1");
		team2 = new TeamInfo(new PartyWithParameters(partyref, parameters),
				profileref);
		team2a = new TeamInfo(new PartyWithParameters(partyref, parameters),
				profileref);

	}

	@Override
	public List<List<TeamInfo>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(team1, team1a),
				Arrays.asList(team2, team2a));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("TeamInfo.\\[\\].",
				"TeamInfo.\\[PartyWithProfile.*party1.*profile1.*\\].*");
	}

	@Test
	public void testSerialize() throws JsonProcessingException {

		ObjectMapper jackson = new ObjectMapper();
		System.out.println(jackson.writeValueAsString(team2));
		assertEquals(asString1, jackson.writeValueAsString(team2));

	}

	@Test
	public void testDeserialize() throws IOException {

		ObjectMapper jackson = new ObjectMapper();
		TeamInfo read = jackson.readValue(asString1, TeamInfo.class);
		assertEquals(team2, read);
	}

}
